#include <stdio.h>

#include <Window.h>
#include <Bitmap.h>
#include <Screen.h>

#include "DefView.h"

//======================================================================
TDefView::TDefView(BRect Frame, uint8 Red, uint8 Green = 1, uint8 Blue = 1)
         :BTextView(Frame, "DefView", Frame, B_FOLLOW_ALL_SIDES,B_WILL_DRAW ),
          FImage(NULL)
{
	SetText("\"Transparency\"\nThis looks transparent even when you type in it.\n");
	MakeEditable(true);
	
    FTintColor.red = Red;
    FTintColor.green = Green;
    FTintColor.blue =  Blue;

	SetColorSpace(B_RGB32);

	rgb_color c = { 255, 255, 255, 0 };
	SetFontAndColor(be_plain_font, B_FONT_ALL, &c); 
}
//----------------------------------------------------------------------
TDefView::~TDefView()
{
	delete FImage;
}
//-------------------------------------------------------------------------------
inline void TDefView::Draw(BRect UpdateRect)
{
	BRect b(Bounds());
	SetDrawingMode(B_OP_COPY);
	DrawBitmap(FImage, ConvertToScreen(b), b);
	
	// The drawing mode changes here are for the antialiasing fix for
	// the text (according to Flax anyway).
	SetDrawingMode(B_OP_OVER);
	BTextView::Draw(b);
}
//----------------------------------------------------------------------
void TDefView::AttachedToWindow()
{
	SetViewColor(B_TRANSPARENT_32_BIT);
	SetHighColor(255, 255, 255);
	SetLowColor(0, 0, 0);
	
	if ( NULL != FImage )
	{
		delete FImage;
	}
	
	BScreen screen;
	Window()->Hide();
	screen.GetBitmap(&FImage);
	if ( B_RGB32 != FImage->ColorSpace() )
	{
		// translate to 32 bit image ...
		BBitmap* temp = new BBitmap(FImage->Bounds(), B_RGB32, true);
		BView aView(FImage->Bounds(), "", 0, B_WILL_DRAW);
		temp->AddChild(&aView);
		aView.Window()->Lock();
		aView.DrawBitmap(FImage);
		aView.Window()->Unlock();
		temp->RemoveChild(&aView);
		delete FImage;
		FImage = temp;
	}
	
	uint32 bitend = FImage->BitsLength() / 4;
	
	// shade the image
	for( uint32 i = 0; i < bitend; i++ )
	{
		rgb_color* color = (rgb_color*)(&((uint32*)(FImage->Bits()))[i]);
		//shading to the color we want
		color->red -= color->red >> FTintColor.red; 
		color->blue -= color->blue >> FTintColor.blue; 
		color->green -= color->green >> FTintColor.green; 
	}
	
	BTextView::Invalidate();
	Window()->Show();
	
	BTextView::AttachedToWindow();
}
//----------------------------------------------------------------------
// Ensure refreshing after pasting
void TDefView::Paste(BClipboard* Clipboard)
{
	BTextView::Paste(Clipboard);
	Draw(Bounds());
}
//----------------------------------------------------------------------
// Not used, but here for fun!
void 
TDefView::KeyUp(const char *bytes, int32 numBytes)
{
	BTextView::KeyUp(bytes, numBytes);
}
//-------------------------------------------------------------------------------
//first call key down and then refresh (in order to avoid insertion problems)
void 
TDefView::KeyDown(const char *bytes, int32 numBytes)
{
	
	BTextView::KeyDown(bytes, numBytes);
	Draw(Bounds());
}	
//----------------------------------------------------------------------
